@@ -1,14 +1,46 @@
 package Any::Moose;
-our $VERSION = '0.11';
+BEGIN {
+  $Any::Moose::VERSION = '0.12';
+}
 # ABSTRACT: use Moose or Mouse modules
 
 use 5.006_002;
 use strict;
 use warnings;
 
-use Carp ();
+# decide which backend to use
+our $PREFERRED;
+do {
+    local $@;
+    if ($ENV{ANY_MOOSE}) {
+        $PREFERRED = $ENV{'ANY_MOOSE'};
+        warn "ANY_MOOSE is not set to Moose or Mouse"
+            unless $PREFERRED eq 'Moose'
+                || $PREFERRED eq 'Mouse';
 
-our $PREFERRED = $ENV{'ANY_MOOSE'};
+        # if we die here, then perl gives "unknown error" which doesn't tell
+        # you what the problem is at all. argh.
+        if ($PREFERRED eq 'Moose' && !eval { require Moose }) {
+            warn "\$ANY_MOOSE is set to Moose but we cannot load it";
+        }
+        elsif ($PREFERRED eq 'Mouse' && !eval { require Mouse }) {
+            warn "\$ANY_MOOSE is set to Mouse but we cannot load it";
+        }
+    }
+    elsif (_is_moose_loaded()) {
+        $PREFERRED = 'Moose';
+    }
+    elsif (eval { require Mouse }) {
+        $PREFERRED = 'Mouse';
+    }
+    elsif (eval { require Moose }) {
+        $PREFERRED = 'Moose';
+    }
+    else {
+        require Carp;
+        warn "Unable to locate Mouse or Moose in INC";
+    }
+};
 
 sub import {
     my $self = shift;
@@ -55,7 +87,12 @@ sub unimport {
            . '$module->unimport();';
         $@;
    };
-   Carp::croak("Cannot unimport Any::Moose: $e") if $e;
+
+   if ($e) {
+        require Carp;
+        Carp::croak("Cannot unimport Any::Moose: $e");
+   }
+
    return;
 }
 
@@ -116,7 +153,10 @@ sub _install_module {
            . '$module->import(@{ $options->{imports} });';
         $@;
     };
-    Carp::croak("Cannot import Any::Moose: $e") if $e;
+    if ($e) {
+        require Carp;
+        Carp::croak("Cannot import Any::Moose: $e");
+    }
     return;
 }
 
@@ -135,45 +175,24 @@ sub any_moose {
 
     return $fragment if $backer =~ /^Moose/;
 
-    if (!$PREFERRED) {
-        local $@;
-        if (_is_moose_loaded()) {
-            $PREFERRED = 'Moose';
-        }
-        elsif (eval { require Mouse }) {
-            $PREFERRED = 'Mouse';
-        }
-        elsif (eval { require Moose }) {
-            $PREFERRED = 'Moose';
-        }
-        else {
-            require Carp;
-            Carp::confess("Unable to locate Mouse or Moose in INC");
-        }
-    }
-
     $fragment =~ s/^Moose/Mouse/ if mouse_is_preferred();
     return $fragment;
 }
 
-sub load_class {
-    my ($class_name) = @_;
-    return Class::MOP::load_class($class_name) if moose_is_preferred();
-    return Mouse::load_class($class_name);
-}
-
-sub is_class_loaded {
-    my ($class_name) = @_;
-    return Class::MOP::is_class_loaded($class_name) if moose_is_preferred();
-    return Mouse::is_class_loaded($class_name);
+for my $name (qw/load_class is_class_loaded class_of get_metaclass_by_name get_all_metaclass_instances get_all_metaclass_names/) {
+    no strict 'refs';
+    *{__PACKAGE__."::$name"} = moose_is_preferred()
+        ? *{"Class::MOP::$name"}
+        : *{"Mouse::Util::$name"};
 }
 
 sub moose_is_preferred { $PREFERRED eq 'Moose' }
 sub mouse_is_preferred { $PREFERRED eq 'Mouse' }
 
-sub _is_moose_loaded { exists $INC{'Class/MOP.pm'} }
+sub _is_moose_loaded { exists $INC{'Moose.pm'} }
 
 sub is_moose_loaded {
+    require Carp;
     Carp::carp("Any::Moose::is_moose_loaded is deprecated. Please use Any::Moose::moose_is_preferred instead");
     goto \&_is_moose_loaded;
 }
@@ -209,15 +228,17 @@ Any::Moose - use Moose or Mouse modules
 
 =head1 VERSION
 
-version 0.11
+version 0.12
 
 =head1 SYNOPSIS
 
+
 =head2 BASIC
 
+
     package Class;
 
-    # uses Moose if it's loaded, Mouse otherwise
+    # uses Moose if it's loaded or demanded, Mouse otherwise
     use Any::Moose;
 
     # cleans the namespace up
@@ -225,6 +246,7 @@ version 0.11
 
 =head2 OTHER MODULES
 
+
     package Other::Class;
     use Any::Moose;
 
@@ -232,8 +254,17 @@ version 0.11
     # Mouse::Util::TypeConstraints otherwise.
     use Any::Moose '::Util::TypeConstraints';
 
+=head2 ROLES
+
+
+    package My::Sorter;
+    use Any::Moose 'Role';
+
+    requires 'cmp';
+
 =head2 COMPLEX USAGE
 
+
     package My::Meta::Class;
     use Any::Moose;
 
@@ -245,7 +276,7 @@ version 0.11
         '::Util' => ['does_role'],
     );
 
-    # uses MouseX::Types
+    # uses MouseX::Types or MooseX::Types
     use Any::Moose 'X::Types';
 
     # gives you the right class name depending on which Mo*se was loaded
@@ -253,15 +284,42 @@ version 0.11
 
 =head1 DESCRIPTION
 
-Actual documentation is forthcoming, once we solidify all the bits of the API.
-The examples above are very likely to continue working.
+
+Though we recommend that people generally use L<Moose>, we accept that Moose
+cannot yet be used for everything everywhere. People generally like the Moose
+sugar, so many people use L<Mouse>, a lightweight replacement for parts of
+Moose.
+
+
+Because Mouse strives for compatibility with Moose, it's easy to substitute one
+for the other. This module facilitates that substitution. By default, Mouse
+will be provided to libraries, unless Moose is already loaded -or-
+explicitly requested by the end-user. The end-user can force the decision
+of which backend to use by setting the environment variable C<ANY_MOOSE> to
+be C<Moose> or C<Mouse>.
+
+
+Note that the decision of which backend to use is made only once, so that if
+Any-Moose picks Mouse, then a third-party library loads Moose, anything else
+that uses Any-Moose will continue to pick Mouse.
+
+
+So, if you have to use L<Mouse>, please be considerate to the Moose fanboys
+(like myself!) and use L<Any-Moose> instead. C<:)>
 
 =head1 SEE ALSO
 
+
 L<Moose>
 
+
 L<Mouse>
 
+
+L<Squirrel> - a deprecated first-stab at Any-Moose-like logic. Its biggest
+fault was in making the decision of which backend to use every time it was
+used, rather than just once.
+
 =head1 AUTHORS
 
   Shawn M Moore <sartak@bestpractical.com>
@@ -272,10 +330,10 @@ L<Mouse>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2009 by Best Practical Solutions.
+This software is copyright (c) 2010 by Best Practical Solutions.
 
 This is free software; you can redistribute it and/or modify it under
-the same terms as perl itself.
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
